/******************************************************************************
 *
 * BixelViewView.cpp - implementation of the CBixelViewView class
 *
 * Portions of this code come from online, the following
 * is the copyright statement of that code:
 *
 * Copyright Notice & Disclaimer
 *
 * Copyright  Alessandro Falappa
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice, author statement
 * appear in all copies of this software and related documentation.
 *
 * If you make enhancement or you discover bugs, please let me
 * know
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF
 * ANY KIND, EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL ALESSANDRO FALAPPA BE LIABLE FOR ANY
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE,
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * http://www.codeguru.com/opengl/oglwrapperclass.shtml
 *
 * The portions of the code that come from this site are the code that
 * lets me use OpenGL in this MDI document created by the wizard
 *
 * In original code, look in GLEnabledView
 *
 *  01/19/02: D. Hazen--File created (wizard)
 *  02/10/02: D. Hazen--Added MDI OpenGL code
 *  02/22/02: D. Hazen--Finalized arcball integration
 *  03/19/02: D. Hazen--Finished adding comments to the file
 *****************************************************************************/

#include "stdafx.h"
#include "BixelView.h"

#include "BixelViewDoc.h"
#include "BixelViewView.h"
#include "ArcBall.h"

#include "Utilities.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView

/////////////////////////////////////////////////////////
// CBixelViewView Constants

// these are used to construct an equilibrated 256 color palette
static unsigned char _threeto8[8] = 
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

static unsigned char _twoto8[4] = 
{
	0, 0x55, 0xaa, 0xff
};

static unsigned char _oneto8[2] = 
{
	0, 255
};

static int defaultOverride[13] = 
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

// Windows Default Palette
static PALETTEENTRY defaultPalEntry[20] = 
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};

/////////////////////////////////////////////////////////////////////////////
// CBixelView Message Map Definition

IMPLEMENT_DYNCREATE(CBixelViewView, CView)

BEGIN_MESSAGE_MAP(CBixelViewView, CView)
	//{{AFX_MSG_MAP(CBixelViewView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_ZOOM_IN, OnZoomIn)
	ON_COMMAND(ID_ZOOM_OUT, OnZoomOut)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView construction/destruction

CBixelViewView::CBixelViewView()
{
	// TODO: add construction code here
}

CBixelViewView::~CBixelViewView()
{
}

BOOL CBixelViewView::PreCreateWindow(CREATESTRUCT& cs)
{
    // these styles are requested by OpenGL
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

    // these styles are meant for a use of this class in a MDI application
	cs.lpszClass = AfxRegisterWndClass(CS_OWNDC | CS_HREDRAW | CS_VREDRAW);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView drawing

void CBixelViewView::OnDraw(CDC* pDC)
{
	CBixelViewDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

    // prepare a semaphore
	static BOOL 	bBusy = FALSE;
    // use the semaphore to enter this critic section
	if(bBusy) return;
	bBusy = TRUE;

    // specify the target DeviceContext of the subsequent OGL calls
	wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

    // clear background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // call the virtual drawing procedure (to be overridden by user)
    if (pDoc->m_CurrentView == NULL) {
        exit(-1);
    }

    // Does the actual drawing of the image on the screen!
	pDoc->m_CurrentView->OnDrawGL(m_ClientRect);

    // execute OGL commands (flush the OGL graphical pipeline)
	glFlush();
    glFinish();

    // if double buffering is used it's time to swap the buffers
	SwapBuffers(m_pCDC->GetSafeHdc());
	
    // turn the semaphore "green"
	bBusy = FALSE;

    // free the target DeviceContext (window)
    wglMakeCurrent(NULL,NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView printing

BOOL CBixelViewView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CBixelViewView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CBixelViewView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView diagnostics

#ifdef _DEBUG
void CBixelViewView::AssertValid() const
{
	CView::AssertValid();
}

void CBixelViewView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBixelViewDoc* CBixelViewView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBixelViewDoc)));
	return (CBixelViewDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBixelViewView message handlers

BOOL CBixelViewView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

int CBixelViewView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
    // This function initializes the settings so that the program
    // will display correctly on all types of machines (from MFC
    // tutorials)

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    // OpenGL rendering context creation
	PIXELFORMATDESCRIPTOR pfd;
    int         n;

    // initialize the private member
	m_pCDC= new CClientDC(this);

    // robust way of choosing the requested video mode
    if (!bSetupPixelFormat()) return 0;
	
    // ask the system if the video mode is supported
    n=::GetPixelFormat(m_pCDC->GetSafeHdc());
    ::DescribePixelFormat(m_pCDC->GetSafeHdc(),n,sizeof(pfd),&pfd);

    // create a palette if the requested video mode has 256 colors (indexed mode)
    CreateRGBPalette();

    // link the Win Device Context with the OGL Rendering Context
    m_hRC = wglCreateContext(m_pCDC->GetSafeHdc());

    // specify the target DeviceContext (window) of the subsequent OGL calls
    wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

    // performs default setting of rendering mode,etc..
	OnCreateGL();

    // free the target DeviceContext (window)
    wglMakeCurrent(NULL,NULL);
	
	return 0;
}

void CBixelViewView::OnCreateGL()
{
    // Do OpenGL initializations here

    /* Some default colors.... */
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glColor3f(0.0, 0.0, 0.0);
    glShadeModel(GL_SMOOTH);

    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
    CLighting light((float)0.7);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light.ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light.diffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, light.position);
	glEnable(GL_LIGHT0);
    glEnable(GL_LIGHTING);

    BuildTexture();

	/* Use a Z-Buffer.... */
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
}

void CBixelViewView::OnDestroy() 
{
	// specify the target DeviceContext (window) of the subsequent OGL calls
    wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

    // release definitely OGL Rendering Context
	if (m_hRC!=NULL) ::wglDeleteContext(m_hRC);

    // Select our palette out of the dc
	CPalette palDefault;
	palDefault.CreateStockObject(DEFAULT_PALETTE);
	m_pCDC->SelectPalette(&palDefault, FALSE);

    // destroy Win Device Context
	if(m_pCDC) delete m_pCDC;

    // finally call the base function
	CView::OnDestroy();	
}

BOOL CBixelViewView::OnEraseBkgnd(CDC* pDC) 
{
	// OGL has his own background erasing so tell Windows to skip
	return TRUE;
}

void CBixelViewView::OnSize(UINT nType, int cx, int cy) 
{
    // Called when window gets resized

	CView::OnSize(nType, cx, cy);
    // when called with a nonzero window:
	if ( 0 < cx && 0 < cy )
	{
        // update the rect and the aspect ratio
		m_ClientRect.right = cx;
		m_ClientRect.bottom = cy;

        // specify the target DeviceContext of the subsequent OGL calls
		wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

        // fix up our projection view with the new window
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();               // Don't forget to clear out the stack!
        gluPerspective(45,((double)cx)/cy,1.0,5.0);  //45 degrees,aspect ratio, znear, zfar

        /* Load in the transformation from the model to the view coords. */
	    glMatrixMode(GL_MODELVIEW);     // From here on out everything effects the model.
	    glLoadIdentity();

        CBixelViewDoc* pDoc = GetDocument();
	    ASSERT_VALID(pDoc);
        CAspectView * view = pDoc->m_CurrentView;

        gluLookAt(view->m_Eye[0],view->m_Eye[1],view->m_Eye[2],
                  view->m_Ctr[0],view->m_Ctr[1],view->m_Ctr[2],
                  view->m_Up[0],view->m_Up[1],view->m_Up[2]);

        // free the target DeviceContext (window)
		wglMakeCurrent(NULL,NULL);

        // force redraw
		Invalidate(TRUE);
	};
}

BOOL CBixelViewView::DestroyWindow() 
{
	// TODO: Add your specialized code here and/or call the base class
	
	return CView::DestroyWindow();
}

// More code to set up the window for support on a variety of machines
BOOL CBixelViewView::bSetupPixelFormat()
{
    // define default desired video mode (pixel format)
	static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),	// size of this pfd
        1,								// version number
        PFD_DRAW_TO_WINDOW |			// support window
        PFD_SUPPORT_OPENGL |			// support OpenGL
        PFD_DOUBLEBUFFER,				// double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
        16,                             // 32-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
// let the user change some parameters if he wants
	BOOL bDoublBuf;
	ColorsNumber cnum;
	ZAccuracy zdepth;
	VideoMode(cnum,zdepth,bDoublBuf);
//set the changes
	if(bDoublBuf) pfd.dwFlags=PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL |PFD_DOUBLEBUFFER;
	else pfd.dwFlags=PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
	switch(cnum)
	{
	case INDEXED: pfd.cColorBits=8;
	case THOUSANDS: pfd.cColorBits=16;
	case MILLIONS: pfd.cColorBits=24;
	case MILLIONS_WITH_TRANSPARENCY: pfd.cColorBits=32;
	};
	switch(zdepth)
	{
	case NORMAL: pfd.cDepthBits=16;
	case ACCURATE: pfd.cDepthBits=32;
	};

// ask the system for such video mode
    ASSERT(m_pCDC != NULL);
    int pixelformat;
	if ( (pixelformat = ChoosePixelFormat(m_pCDC->GetSafeHdc(), &pfd)) == 0 )
    {
        AfxMessageBox("ChoosePixelFormat failed");
        return FALSE;
    }

// try to set this video mode    
	if (SetPixelFormat(m_pCDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
    {
// the requested video mode is not available so get a default one
        pixelformat = 1;	
		if (DescribePixelFormat(m_pCDC->GetSafeHdc(), pixelformat, sizeof(PIXELFORMATDESCRIPTOR), &pfd)==0)
		{
// neither the requested nor the default are available: fail
			AfxMessageBox("SetPixelFormat failed (no OpenGL compatible video mode)");
			return FALSE;
		}
    }

    return TRUE;
}

// More code to set up the window for support on a variety of machines
void CBixelViewView::CreateRGBPalette()
{
    PIXELFORMATDESCRIPTOR pfd;
    LOGPALETTE *pPal;
    int n, i;

    // get the initially choosen video mode
	n = ::GetPixelFormat(m_pCDC->GetSafeHdc());
    ::DescribePixelFormat(m_pCDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

    // if is an indexed one...
    if (pfd.dwFlags & PFD_NEED_PALETTE)
    {
        // ... construct an equilibrated palette (3 red bits, 3 green bits, 2 blue bits)
        // NOTE: this code is integrally taken from MFC example Cube
		n = 1 << pfd.cColorBits;
        pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

        ASSERT(pPal != NULL);

        pPal->palVersion = 0x300;
        pPal->palNumEntries = n;
        for (i=0; i<n; i++)
        {
            pPal->palPalEntry[i].peRed=ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
            pPal->palPalEntry[i].peGreen=ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
            pPal->palPalEntry[i].peBlue=ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
            pPal->palPalEntry[i].peFlags=0;
        }

        // fix up the palette to include the default Windows palette
        if ((pfd.cColorBits == 8)                           &&
            (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
            (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
            (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
           )
        {
			for (i = 1 ; i <= 12 ; i++)
                pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
        }

        m_CurrentPalette.CreatePalette(pPal);
        delete pPal;

        // set the palette
        m_pOldPalette=m_pCDC->SelectPalette(&m_CurrentPalette, FALSE);
        m_pCDC->RealizePalette();
    }
}

// More code to set up the window for support on a variety of machines
void CBixelViewView::VideoMode(ColorsNumber &c, ZAccuracy &z, BOOL &dbuf)
{
// set default videomode
	c=MILLIONS;
	z=NORMAL;
	dbuf=TRUE;
}

// More code to set up the window for support on a variety of machines
unsigned char CBixelViewView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
    unsigned char val;

    val = (unsigned char) (i >> shift);
    switch (nbits) 
	{

    case 1:
        val &= 0x1;
        return _oneto8[val];
    case 2:
        val &= 0x3;
        return _twoto8[val];
    case 3:
        val &= 0x7;
        return _threeto8[val];

    default:
        return 0;
    }
}

// Sets up the display list for a texture (which gets drawn in
// CAspectView::OnDrawGL)
void CBixelViewView::BuildTexture()
{
    GetDocument()->m_CurrentView->m_Texture.ReadFile("Flower.bmp");
	GetDocument()->m_CurrentView->m_Texture.BGRtoRGB();

    ::glNewList(1,GL_COMPILE_AND_EXECUTE);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
    glEnable(GL_TEXTURE_2D);
    glShadeModel(GL_SMOOTH);

    if(GetDocument()->m_CurrentView->m_Texture.GetData() != NULL)
        glTexImage2D(GL_TEXTURE_2D,0,3,
		             GetDocument()->m_CurrentView->m_Texture.GetWidth(),
					 GetDocument()->m_CurrentView->m_Texture.GetHeight(),0,
					 GL_RGB,GL_UNSIGNED_BYTE,
					 GetDocument()->m_CurrentView->m_Texture.GetData());

    // defines the number of tiles of the image (1 means just the image, no tiling)
	float xt = 1.0f;
	glBegin(GL_POLYGON);
        // map texture coords to the vertices in 3 space
        glTexCoord2f(0.0, 0.0);
        glVertex3f(1.0,1.0,0.0);  // glVertex3d( x,  x, x);
        glTexCoord2f(0.0, xt);
        glVertex3f(-1.0,1.0,0.0);  // glVertex3d(-x,  x, x);
        glTexCoord2f(xt, xt);  
        glVertex3f(-1.0,-1.0,0.0);  // glVertex3d(-x, -x, x);
        glTexCoord2f(xt, 0.0);
        glVertex3f(1.0,-1.0,0.0);  // glVertex3d( x, -x, x);
        glNormal3d(0.0,0.0,1.0);		 
	glEnd();

	::glDisable(GL_TEXTURE_2D);

	::glEndList();
}

// Calls arcball code to begin a rotation
void CBixelViewView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CArcBall * arcBall = GetDocument()->m_CurrentView->m_ArcBall;

    RECT winRect;
    GetWindowRect(&winRect);
    
    CQuat vect = arcBall->Convert(point.x, point.y, winRect.right - winRect.left, winRect.bottom - winRect.top);
    arcBall->BallMouse(vect);
    arcBall->BallBeginDrag();
    arcBall->BallUpdate();
    Invalidate();
    // Capture mouse movement outside of the window
    SetCapture();

	CView::OnLButtonDown(nFlags, point);
}

// Calls arcball code for mouse up events
void CBixelViewView::OnLButtonUp(UINT nFlags, CPoint point) 
{
    if (GetCapture() == this) {
        ::ReleaseCapture();

        CArcBall * arcBall = GetDocument()->m_CurrentView->m_ArcBall;

        RECT winRect;
        GetWindowRect(&winRect);
    
        CQuat vect = arcBall->Convert(point.x, point.y, winRect.right - winRect.left, winRect.bottom - winRect.top);
        arcBall->BallMouse(vect);
        arcBall->BallEndDrag();
        arcBall->BallUpdate();
        arcBall->BallValue();
        Invalidate();
    }

	CView::OnLButtonUp(nFlags, point);
}

// Calls arcball code for mouse movement events (to keep the 
// current rotation drag up to date and to do rotations as the
// mouse is dragging)
void CBixelViewView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CArcBall * arcBall = GetDocument()->m_CurrentView->m_ArcBall;

    if (arcBall->GetDragging()) 
    {   
        RECT winRect;
        GetWindowRect(&winRect);

        CQuat vect = arcBall->Convert(point.x, point.y, winRect.right - winRect.left, winRect.bottom - winRect.top);

        arcBall->BallMouse(vect);                    // set the mouse position
        arcBall->BallUpdate();                // alters the internal state of the arcball
        arcBall->BallValue();  // reads the matrix from the arcball
        Invalidate();
    }

	CView::OnMouseMove(nFlags, point);
}

// Code that zooms in the display
void CBixelViewView::OnZoomIn() 
{
    // specify the target DeviceContext of the subsequent OGL calls
	wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

    CBixelViewDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);

    // do the actual zooming in
	pDoc->m_CurrentView->OnZoomIn();
    glMatrixMode(GL_MODELVIEW);     // From here on out everything effects the model.
	glLoadIdentity();

    CAspectView * view = pDoc->m_CurrentView;

    // update the GL matrices
    gluLookAt(view->m_Eye[0],view->m_Eye[1],view->m_Eye[2],
              view->m_Ctr[0],view->m_Ctr[1],view->m_Ctr[2],
              view->m_Up[0],view->m_Up[1],view->m_Up[2]);

    Invalidate(TRUE);
    // free the target DeviceContext (window)
	wglMakeCurrent(NULL,NULL);
}

// Zoom out display
void CBixelViewView::OnZoomOut() 
{
    // specify the target DeviceContext of the subsequent OGL calls
	wglMakeCurrent(m_pCDC->GetSafeHdc(), m_hRC);

	CBixelViewDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);

	pDoc->m_CurrentView->OnZoomOut();
    glMatrixMode(GL_MODELVIEW);     // From here on out everything effects the model.
	glLoadIdentity();

    CAspectView * view = pDoc->m_CurrentView;

    // Update the GL matrices
    gluLookAt(view->m_Eye[0],view->m_Eye[1],view->m_Eye[2],
              view->m_Ctr[0],view->m_Ctr[1],view->m_Ctr[2],
              view->m_Up[0],view->m_Up[1],view->m_Up[2]);

    // free the target DeviceContext (window)
	wglMakeCurrent(NULL,NULL);
    Invalidate(TRUE);
}
