/******************************************************************************
 *
 * BixelViewView.h - implementation of the CBixelViewView class
 *
 * Portions of this code come from online, the following
 * is the copyright statement of that code:
 *
 * Copyright Notice & Disclaimer
 *
 * Copyright  Alessandro Falappa
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice, author statement
 * appear in all copies of this software and related documentation.
 *
 * If you make enhancement or you discover bugs, please let me
 * know
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF
 * ANY KIND, EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL ALESSANDRO FALAPPA BE LIABLE FOR ANY
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA
 * OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE,
 * AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * http://www.codeguru.com/opengl/oglwrapperclass.shtml
 *
 * The portions of the code that come from this site are the code that
 * lets me use OpenGL in this MDI document created by the wizard
 *
 *  01/19/02: D. Hazen--File created (wizard)
 *  02/10/02: D. Hazen--Added MDI OpenGL code
 *  02/22/02: D. Hazen--Finalized arcball integration
 *  03/19/02: D. Hazen--Finished adding comments to the file
 *****************************************************************************/

#if !defined(AFX_BIXELVIEWVIEW_H__5C6F6E4F_3299_11D6_9F9F_00105A17CA6F__INCLUDED_)
#define AFX_BIXELVIEWVIEW_H__5C6F6E4F_3299_11D6_9F9F_00105A17CA6F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define FOV_ANGLE 45

// enums used to try to ensure proper execution on different machines
enum ZAccuracy{NORMAL,ACCURATE};
enum ColorsNumber{INDEXED,THOUSANDS,MILLIONS,MILLIONS_WITH_TRANSPARENCY};

// I would suggest that you look at the Scribble tutorial if you don't
// understand the View/Document style of programming.

// If you are confused about where to add code, I would say that this
// is most likely a starting point as this class basically represents
// the current window you have open (this is a Multiple Document Interface
// MDI program, meaning multiple windows can be open at once (like MS Word))
// Thus, most code to change what the user sees on the screen in a window
// goes here.  If it doesn't go here, look at the BixelViewDoc files as
// they are the other place where code may be needed.

class CBixelViewView : public CView
{
protected: // create from serialization only
	CBixelViewView();
	DECLARE_DYNCREATE(CBixelViewView)

// member definitions
private:
    CDC* m_pCDC;	// WinGDI Device Context
	HGLRC m_hRC;	// OpenGL Rendering Context
    CPalette m_CurrentPalette; // palettes
	CPalette* m_pOldPalette;
    CRect m_ClientRect;    // client area size

// Attributes
public:
	CBixelViewDoc* GetDocument();
    void VideoMode(ColorsNumber &c, ZAccuracy &z, BOOL &dbuf);

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBixelViewView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	virtual BOOL DestroyWindow();
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	void OnCreateGL();
	virtual ~CBixelViewView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
    // builds a texture from an image which is then displayed on the screen
	void BuildTexture();
    // the following 3 functions are basic MDI functions from the MFC
    // tutorials that just work to ensure proper functioning of this 
    // program on systems with different specifications
	void CreateRGBPalette();
	BOOL bSetupPixelFormat();
    unsigned char ComponentFromIndex(int i, UINT nbits, UINT shift);
	//{{AFX_MSG(CBixelViewView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnZoomIn();
	afx_msg void OnZoomOut();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in BixelViewView.cpp
inline CBixelViewDoc* CBixelViewView::GetDocument()
   { return (CBixelViewDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BIXELVIEWVIEW_H__5C6F6E4F_3299_11D6_9F9F_00105A17CA6F__INCLUDED_)
